@@ -17,8 +17,14 @@ def _available_backends():
 
     if _available_backends_list is None:
         _available_backends_list = [
-            backend.load(require=False)
-            for backend in pkg_resources.iter_entry_points(
+            # DeprecatedIn16
+            # setuptools 11.3 deprecated support for the require parameter to
+            # load(), and introduced the new resolve() method instead.
+            # We previously removed this fallback, but users are having issues
+            # where Python loads an older setuptools due to various syspath
+            # weirdness.
+            ep.resolve() if hasattr(ep, "resolve") else ep.load(require=False)
+            for ep in pkg_resources.iter_entry_points(
                 "cryptography.backends"
             )
         ]
